/*
************************************************************************
*
*   ExCenter.c - Center command
*
*   Copyright (c) 1994
*
*   ETH Zuerich
*   Institut fuer Molekularbiologie und Biophysik
*   ETH-Hoenggerberg
*   CH-8093 Zuerich
*
*   SPECTROSPIN AG
*   Industriestr. 26
*   CH-8117 Faellanden
*
*   All Rights Reserved
*
*   Date of last modification : 95/03/10
*   Pathname of SCCS file     : /sgiext/molmol/src/cmdrep/SCCS/s.ExCenter.c
*   SCCS identification       : 1.2
*
************************************************************************
*/

#include <cmd_rep.h>

#include <mat_vec.h>
#include <data_hand.h>
#include <data_sel.h>
#include <graph_draw.h>

static void
molCenter(DhMolP molP, void *clientData)
{
  Vec3 cent, transV, oldRotV, newRotV;
  Vec4 v4;
  Mat4 rotM;

  Vec3Copy(cent, clientData);

  DhMolGetTransVect(molP, transV);
  DhMolSetTransVect(molP, cent);

  DhMolGetRotMat(molP, rotM);
  DhMolGetRotPoint(molP, oldRotV);

  Vec3Copy(newRotV, cent);
  Vec3Sub(newRotV, transV);
  Vec3To4(v4, newRotV);
  Mat4Transp(rotM);
  Mat4VecMult(v4, rotM);
  Vec4To3(newRotV, v4);
  Vec3Add(newRotV, oldRotV);

  DhMolSetRotPoint(molP, newRotV);
}

ErrCode
ExCenter(char *cmd)
{
  int selNo;
  DhAtomP atomP;
  Vec3 cent;

  selNo = SelAtomGet(&atomP, 1);
  if (selNo != 1) {
    CipSetError("exactly 1 atom must be selected");
    return EC_ERROR;
  }

  DhAtomGetCoordTransf(atomP, cent);

  DhApplyMol(PropGetRef(PROP_MOVABLE, FALSE), molCenter, cent);

  GraphRedrawNeeded();

  return EC_OK;
}
